@@ -1,6 +1,6 @@
 ;;; nnshimbun.el --- interfacing with web newspapers
 
-;; Copyright (C) 2000-2012, 2016 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2000-2012, 2016, 2017 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;          ARISAWA Akihiro    <ari@mbf.sphere.ne.jp>,
@@ -106,8 +106,7 @@ Note that smaller values may spoil the `prefetch-articles' feature,
 since nnshimbun uses the backlog to keep the prefetched articles."
   :group 'nnshimbun
   :type '(choice (const :tag "Off" nil)
-		 (integer :format "%t: %v\n"
-			  :size 0)
+		 (integer :format "%t: %v\n")
 		 (sexp :format "All\n"
 		       :match (lambda (widget value)
 				(and value (not (numberp value))))
@@ -932,8 +931,6 @@ shimbun group."
 		   alist nil t
 		   (car (delete "" nnshimbun-server-history))
 		   'nnshimbun-server-history))
-     (static-unless (featurep 'xemacs)
-       (setq server (string-make-unibyte server)))
      (if (assoc server alist)
 	 (let ((shimbun (shimbun-open server)))
 	   (setq group (completing-read
