@@ -1,6 +1,6 @@
 ;;; shimbun.el --- interfacing with web newspapers -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001-2013 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 2001-2014, 2017 Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;         Akihiro Arisawa    <ari@mbf.sphere.ne.jp>,
@@ -73,6 +73,13 @@
 (eval-when-compile (require 'cl))
 (eval-when-compile (require 'static))
 
+;; Make edebug work for the static-* macros in Emacs 24.4+.
+(def-edebug-spec static-cond (&rest (&rest def-form)))
+(def-edebug-spec static-defconst (&define name def-body [&optional def-body]))
+(def-edebug-spec static-if (&rest def-form))
+(def-edebug-spec static-unless (&rest def-form))
+(def-edebug-spec static-when (&rest def-form))
+
 (require 'mcharset)
 (require 'eword-encode)
 (require 'luna)
@@ -111,28 +118,30 @@
 dJrT4Cd<Ls?U!G4}0S%FA~KegR;YZWieoc%`|$4M\\\"i*2avWm?"
   "*Default X-Face field for shimbun."
   :group 'shimbun
-  :type '(string :format "%{%t%}:\n%v" :size 0))
+  :type '(string :format "%{%t%}:\n%v"))
 
 (defcustom shimbun-server-additional-path nil
   "*List of additional directories to search for shimbun servers."
   :group 'shimbun
-  :type '(repeat (directory :format "%t: %v\n" :size 0)))
+  :type '(repeat (directory :format "%t: %v\n")))
 
 (defcustom shimbun-checking-new-news-format "Checking new news on #S for #g"
   "*Format string used to show a progress message while chacking new news.
 See `shimbun-message' for the special format specifiers."
   :group 'shimbun
-  :type '(string :format "%{%t%}:\n%v" :size 0))
+  :type '(string :format "%{%t%}:\n%v"))
 
 (defcustom shimbun-verbose t
-  "*Flag controls whether shimbun should be verbose.
-If it is non-nil, the `w3m-verbose' variable will be bound to nil
-while shimbun is waiting for a server's response."
+  "If non-nil, `shimbun-message' will display echo messages.
+If it is nil, messages will be neither displayed nor logged into the
+\"*Messages*\" buffer.  Note that the meaning of this variable differs
+from `w3m-verbose'.  See also `shimbun-message-enable-logging'."
   :group 'shimbun
   :type 'boolean)
 
 (defcustom shimbun-message-enable-logging nil
-  "*Non-nil means preserve echo messages in the *Message* buffer."
+  "If non-nil, `shimbun-message' logs echo messages in *Messages* buffer.
+See also `shimbun-verbose'."
   :group 'shimbun
   :type 'boolean)
 
@@ -246,7 +255,9 @@ Default is the value of `w3m-default-save-directory'."
 (defun shimbun-retrieve-url (url &optional no-cache no-decode referer)
   "Rertrieve URL contents and insert to current buffer.
 Return content-type of URL as string when retrieval succeeded."
-  (let (type charset fname)
+  (let ((w3m-message-silent (not shimbun-verbose))
+	(w3m-verbose shimbun-message-enable-logging)
+	type charset fname)
     (if (and url
 	     shimbun-use-local
 	     shimbun-local-path
@@ -261,9 +272,8 @@ Return content-type of URL as string when retrieval succeeded."
 	    (insert-file-contents fname))
 	  (when (re-search-forward "^$" nil t)
 	    (let ((pos (match-beginning 0)))
-	      (re-search-backward
-	       "^Content-Type: \\(.*?\\)\\(?:[ ;]+\\|$\\)\\(charset=\\(.*\\)\\)?"
-	       nil t)
+	      (re-search-backward "\
+^Content-Type: \\(.*?\\)\\(?:[ ;]+\\|$\\)\\(charset=\\(.*\\)\\)?" nil t)
 	      (setq type (match-string 1)
 		    charset (match-string 3))
 	      (delete-region (point-min) pos))))
@@ -761,7 +771,7 @@ you want to use no database."
 	  (const :tag "Use no database" never)
 	  (const :tag "Use BBDB" shimbun-bbdb-get-x-face)
 	  (const :tag "Use LSDB" shimbun-lsdb-get-x-face)
-	  (function :format "User defined function: %v\n" :size 0)))
+	  (function :format "User defined function: %v\n")))
 
 (defun shimbun-header-insert (shimbun header)
   (let ((from (shimbun-header-from header))
@@ -994,7 +1004,8 @@ Bind it to nil per shimbun if the refresh brings unwanted page.")
   (shimbun-message shimbun (concat shimbun-checking-new-news-format "..."))
   (prog1
       (with-temp-buffer
-	(let ((w3m-verbose (if shimbun-verbose nil w3m-verbose))
+	(let ((w3m-message-silent (not shimbun-verbose))
+	      (w3m-verbose shimbun-message-enable-logging)
 	      headers)
 	  (shimbun-headers-1 shimbun (shimbun-index-url shimbun))
 	  (setq headers (shimbun-get-headers shimbun range))
@@ -1459,8 +1470,8 @@ format specifiers:
 
 Use ## to put a single # into the output.  If `shimbun-verbose' is nil,
 it will run silently.  The `shimbun-message-enable-logging' variable
-controls whether this function should preserve a message in the
-*Messages* buffer."
+controls whether this function should log messages in the \"*Messages*\"
+buffer."
   (let (specifier)
     (with-temp-buffer
       (static-unless (featurep 'xemacs)
@@ -1493,31 +1504,6 @@ controls whether this function should preserve a message in the
 	    (apply 'message fmt args))))
     (apply 'format fmt args)))
 
-(defun shimbun-break-long-japanese-lines (&optional shimbun)
-  "Break long Japanese lines in an article.
-Article should be charset decoded html data.  If SHIMBUN is given,
-this function will narrow the buffer to just an article using the
-shimbun class variables `content-start' and `content-end'.  Otherwise,
-it considers the buffer has already been narrowed to an article."
-  (save-restriction
-    (when shimbun
-      (goto-char (point-min))
-      (let ((case-fold-search t)
-	    start)
-	(when (and (re-search-forward (shimbun-content-start shimbun) nil t)
-		   (setq start (point))
-		   (re-search-forward (shimbun-content-end shimbun) nil t))
-	  (narrow-to-region start (match-beginning 0)))))
-    (goto-char (point-min))
-    (while (re-search-forward
-	    "<p[\t\n ]+[^>]*>\\|</p>\\|\\([$B!"!#!K!W(B]+\\)\\(\\cj\\)?"
-	    nil t)
-      (if (match-beginning 2)
-	  (replace-match "\\1\n\\2")
-	(unless (eolp)
-	  (insert "\n")))))
-  (goto-char (point-min)))
-
 (defmacro shimbun-with-narrowed-article (shimbun &rest forms)
   "Narrow to the article in the buffer and evaluate FORMS."
   `(progn
