@@ -1,6 +1,7 @@
 ;;; w3m-filter.el --- filtering utility of advertisements on WEB sites -*- coding: utf-8 -*-
 
-;; Copyright (C) 2001-2008, 2012, 2013 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001-2008, 2012-2015, 2017
+;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 ;; Keywords: w3m, WWW, hypermedia
@@ -64,23 +65,29 @@
       "テーブル内の <tfoot> を <tbody> の後に描画します")
      ""
      w3m-filter-fix-tfoot-rendering)
+    (t
+     "Filter top and bottom cruft for rt.com"
+     "\\`https://www.rt\\.com/"
+     w3m-filter-rt)
+    (t
+     "Filter for slashdot"
+     "\\`http[s]?://\\([a-z]+\\.\\)?slashdot\\.org/"
+     w3m-filter-slashdot)
     (nil
      ("Remove garbage in http://www.geocities.co.jp/*"
       "http://www.geocities.co.jp/* でゴミを取り除きます")
      "\\`http://www\\.geocities\\.co\\.jp/"
-     (w3m-filter-delete-regions
-      "<DIV ALIGN=CENTER>\n<!--*/GeoGuide/*-->" "<!--*/GeoGuide/*-->\n</DIV>"))
+     w3m-filter-geocities-remove-garbage)
     (nil
      ("Remove ADV in http://*.hp.infoseek.co.jp/*"
       "http://*.hp.infoseek.co.jp/* で広告を取り除きます")
      "\\`http://[a-z]+\\.hp\\.infoseek\\.co\\.jp/"
-     (w3m-filter-delete-regions "<!-- start AD -->" "<!-- end AD -->"))
+     w3m-filter-infoseek-remove-ads)
     (nil
      ("Remove ADV in http://linux.ascii24.com/linux/*"
       "http://linux.ascii24.com/linux/* で広告を取り除きます")
      "\\`http://linux\\.ascii24\\.com/linux/"
-     (w3m-filter-delete-regions
-      "<!-- DAC CHANNEL AD START -->" "<!-- DAC CHANNEL AD END -->"))
+     w3m-filter-ascii24-remove-ads)
     (nil
      "A filter for Google"
      "\\`http://\\(www\\|images\\|news\\|maps\\|groups\\)\\.google\\."
@@ -225,19 +232,19 @@ This variable is semi-obsolete; use `w3m-filter-configuration' instead."
   :group 'w3m
   :type '(repeat
 	  (group :format "%v" :indent 2
-		 (regexp :format "Regexp: %v\n" :value ".*" :size 0)
+		 (regexp :format "Regexp: %v\n" :value ".*")
 		 (choice
 		  :tag "Filtering Rule"
 		  (group :inline t
 			 :tag "Delete regions surrounded with these patterns"
 			 (const :format "Function: %v\n"
 				w3m-filter-delete-regions)
-			 (string :format "Start: %v\n" :size 0
+			 (string :format "Start: %v\n"
 				 :value "not a regexp")
-			 (string :format "  End: %v\n" :size 0
+			 (string :format "  End: %v\n"
 				 :value "not a regexp"))
 		  (function :tag "Filter with a user defined function"
-			    :format "Function: %v\n" :size 0)))))
+			    :format "Function: %v\n")))))
 
 (defcustom w3m-filter-google-use-utf8
   (or (featurep 'un-define) (fboundp 'utf-translate-cjk-mode)
@@ -273,7 +280,7 @@ This variable is semi-obsolete; use `w3m-filter-configuration' instead."
    "/\\([0-9]+\\)")
   "*Regexp to extract ASIN number for Amazon."
   :group 'w3m
-  :type '(string :size 0))
+  :type 'string)
 
 (defcustom w3m-filter-amazon-short-url-bottom nil
   "*Amazon short URLs insert bottom position."
@@ -296,22 +303,93 @@ This variable is semi-obsolete; use `w3m-filter-configuration' instead."
       (when (string-match (car elem) url)
 	(apply (cadr elem) url (cddr elem))))))
 
-(defun w3m-filter-delete-regions (url start end)
-  "Delete regions surrounded with a START pattern and an END pattern."
-  (goto-char (point-min))
-  (let (p (i 0))
-    (while (and (search-forward start nil t)
-		(setq p (match-beginning 0))
-		(search-forward end nil t))
-      (delete-region p (match-end 0))
-      (incf i))
-    (> i 0)))
+(defvar w3m-filter-selection-history nil)
 
-(defun w3m-filter-replace-regexp (url regexp to-string)
-  "Replace all occurrences of REGEXP with TO-STRING."
-  (goto-char (point-min))
-  (while (re-search-forward regexp nil t)
-    (replace-match to-string nil nil)))
+;;;###autoload
+(defun w3m-toggle-filtering (arg)
+  "Toggle whether web pages will have their html modified by w3m's \
+filters before being rendered.
+When called with a prefix argument, prompt for a single filter to
+toggle with completion (a function toggled last will first appear)."
+  (interactive "P")
+  (if (not arg)
+      ;; toggle state for all filters
+      (progn
+	(setq w3m-use-filter (not w3m-use-filter))
+	(message (concat
+		  "web page filtering now "
+		  (if w3m-use-filter "enabled" "disabled"))))
+    ;; the remainder of this function if for the case of toggling
+    ;; an individual filter
+    (let* ((selection-list (delq nil (mapcar
+				      (lambda (elem)
+					(when (and (symbolp (nth 3 elem))
+						   (fboundp (nth 3 elem)))
+					  (symbol-name (nth 3 elem))))
+				      w3m-filter-configuration)))
+	   (choice (completing-read
+		    "Enter filter name: " selection-list nil t
+		    (or (car w3m-filter-selection-history)
+			(car selection-list))
+		    'w3m-filter-selection-history))
+	   (filters w3m-filter-configuration)
+	   elem)
+      (unless (string= "" choice)
+	(setq choice (intern choice))
+	(while (setq elem (pop filters))
+	  (when (eq choice (nth 3 elem))
+	    (setq filters nil)
+	    (setcar elem (not (car elem)))
+	    (when (car elem)
+	      (setq w3m-use-filter t))
+	    (message "filter `%s' now %s"
+		     choice
+		     (if (car elem) "enabled" "disabled"))))))))
+
+(defmacro w3m-filter-delete-regions (url start end
+					 &optional without-start without-end
+					 use-regex start-pos end-pos count)
+  "Delete regions surrounded with a START pattern and an END pattern.
+Return t if at least one region is deleted.
+If WITHOUT-START is non-nil, do not delete the START pattern.
+If WITHOUT-END is non-nil, do not delete the the END strings.
+If USE-REGEX is non-nil, treat START and END as regular expressions.
+START-POS is a position from which to begin deletions.
+END-POS is a position at which to stop deletions.
+COUNT is the maximum number of deletions to make."
+  `(let (p (i 0))
+     (goto-char ,(or start-pos '(point-min)))
+     (while (and ,@(if count `((< i ,count)))
+		 ,(if use-regex
+		      `(re-search-forward ,start ,end-pos t)
+		    `(search-forward ,start ,end-pos t))
+		 (setq p ,(if without-start
+			      '(match-end 0)
+			    '(match-beginning 0)))
+		 ,(if use-regex
+		      `(re-search-forward ,end ,end-pos t)
+		    `(search-forward ,end ,end-pos t)))
+       (delete-region p ,(if without-end
+			     '(match-beginning 0)
+			   '(match-end 0)))
+       (setq i (1+ i)))
+     (> i 0)))
+
+(defmacro w3m-filter-replace-regexp (url regexp to-string
+					 &optional start-pos end-pos count)
+  "Replace all occurrences of REGEXP with TO-STRING.
+Return t if at least one replacement is made.
+Optional START-POS, END-POS, and COUNT limit the scope of
+the replacements."
+  `(let ((i 0))
+     (goto-char ,(or start-pos '(point-min)))
+     (while ,(if count
+		 `(and (< i ,count)
+		       #1=(re-search-forward ,regexp ,end-pos t))
+	       `#1#)
+       (replace-match ,to-string nil nil)
+       (setq i (1+ i)))
+     (> i 0)))
 
 ;; Filter functions:
 (defun w3m-filter-google-click-tracking (url)
@@ -370,83 +448,97 @@ href=\"\\)\\(?:[^\"]+\\)?/\\(?:imgres\\?imgurl\\|url\\?\\(?:q\\|url\\)\\)=\
 	  (goto-char (point-max)))))))
 
 (defun w3m-filter-add-name-anchors (url)
-  ;;  cf. [emacs-w3m:11153]
+  ;;  cf. [emacs-w3m:11153], [emacs-w3m:12339], [emacs-w3m:12422],
+  ;; [emacs-w3m:12812], [emacs-w3m:12830]
   "Add name anchors that w3m can handle.
 This function adds ``<a name=\"FOO_BAR\"></a>'' in front of
 ``<TAG ... id=\"FOO_BAR\" ...>FOO BAR</TAG>'' in the current buffer."
-  (let ((case-fold-search t) names st nd name)
+  (let ((case-fold-search t)
+	(maxregexps 10)
+	names regexp i st nd)
     (goto-char (point-min))
     (while (re-search-forward "<a[\t\n\r ]+\\(?:[^\t\n\r >]+[\t\n\r ]+\\)*\
-href=\"#\\([a-z][-.0-9:_a-z]*\\)\"" nil t)
+href=\"#\\([^\"]+\\)\"" nil t)
       (add-to-list 'names (match-string 1)))
-    (when names
-      (setq names (concat "<\\(?:[^\t\n\r >]+\\)\
-\[\t\n\r ]+\\(?:[^\t\n\r >]+[\t\n\r ]+\\)*[Ii][Dd]=\"\\("
-			  (mapconcat 'regexp-quote names "\\|")
-			  "\\)\"")
-	    case-fold-search nil)
+    (setq case-fold-search nil)
+    (while names
+      (setq regexp "[\t\n\r ]+[Ii][Dd]=\"\\("
+	    i maxregexps)
+      (while (and names (> i 0))
+	(setq regexp (concat regexp (regexp-quote (pop names)) "\\|")
+	      i (1- i)))
+      (setq regexp (concat (substring regexp 0 -1) ")\""))
       (goto-char (point-min))
-      (while (re-search-forward names nil t)
-	(goto-char (setq st (match-beginning 0)))
-	(setq nd (match-end 0)
-	      name (match-string 1))
-	(insert "<a name=" name "></a>")
-	(goto-char (+ nd (- (point) st)))))))
+      (while (re-search-forward "<[^>]+>" nil t)
+	(setq st (match-beginning 0)
+	      nd (match-end 0))
+	(goto-char st)
+	(if (re-search-forward regexp nd t)
+	    (progn
+	      (goto-char st)
+	      (insert "<a name=" (match-string 1) "></a>")
+	      (goto-char (+ nd (- (point) st))))
+	  (goto-char nd))))))
 
 (defun w3m-filter-subst-disabled-with-readonly (url)
   ;;  cf. [emacs-w3m:12146] [emacs-w3m:12222]
   "Substitute disabled attr with readonly attr in forms."
-  (let ((case-fold-search t) st nd val default)
+  (let ((case-fold-search t) st opt nd val default)
     (goto-char (point-min))
     (while (re-search-forward "\
-<\\(?:input\\|\\(option\\)\\|textarea\\)\
-\\(?:[\t\n ]+[^\t\n >]+\\)*[\t\n ]\
-\\(?:\\(disabled\\(=\"[^\"]+\"\\)?\\)\\|\\(readonly\\(?:=\"[^\"]+\"\\)?\\)\\)\
-\\(?:[\t\n ]+[^\t\n >]+\\)*[\t\n /]*>" nil t)
+<\\(?:input\\|\\(option\\)\\|textarea\\)[\t\n ]" nil t)
       (setq st (match-beginning 0)
-	    nd (match-end 0)
-	    val (if (match-beginning 2)
-		    (if (match-beginning 3)
-			"readonly=\"readonly\""
-		      "readonly")
-		  (match-string 4)))
-      (if (match-beginning 1)
-	  ;; Unfortunately w3m doesn't support readonly attr in select forms,
-	  ;; so we replace them with read-only input forms.
-	  (if (and (re-search-backward "<select[\t\n ]" nil t)
-		   (w3m-end-of-tag "select")
-		   (< st (match-end 0)))
-	      (save-restriction
-		(narrow-to-region (match-beginning 0) (match-end 0))
-		(goto-char (+ (match-beginning 0) 8))
-		(w3m-parse-attributes (id name)
-		  (if (and id name)
-		      (progn
-			(goto-char (point-min))
-			(setq default
-			      (when (re-search-forward "<option\
+	    opt (match-beginning 1))
+      (when (and (search-forward ">" nil t)
+		 (progn
+		   (setq nd (match-end 0))
+		   (goto-char (1+ st))
+		   (re-search-forward "[\t\n ]\
+\\(?:\\(disabled\\(=\"[^\"]+\"\\)?\\)\\|\\(readonly\\(?:=\"[^\"]+\"\\)?\\)\\)"
+				      nd t)))
+	(setq val (if (match-beginning 1)
+		      (if (match-beginning 2)
+			  "readonly=\"readonly\""
+			"readonly")
+		    (match-string 3)))
+	(if opt
+	    ;; Unfortunately w3m doesn't support readonly attr in `select'
+	    ;; forms, so we replace them with read-only input forms.
+	    (if (and (re-search-backward "<select[\t\n ]" nil t)
+		     (w3m-end-of-tag "select")
+		     (< st (match-end 0)))
+		(save-restriction
+		  (narrow-to-region (match-beginning 0) (match-end 0))
+		  (goto-char (+ (match-beginning 0) 8))
+		  (w3m-parse-attributes (id name)
+		    (if (and id name)
+			(progn
+			  (goto-char (point-min))
+			  (setq default
+				(when (re-search-forward "<option\
 \\(?:[\t\n ]+[^\t\n >]+\\)*[\t\n ]selected\\(?:=\"[^\"]+\"\\)?\
 \\(?:[\t\n ]+[^\t\n >]+\\)*[\t\n /]*>[\t\n ]*\\([^<]+\\)" nil t)
-				(goto-char (match-end 1))
-				(skip-chars-backward "\t\n ")
-				(buffer-substring (match-beginning 1) (point))))
-			(delete-region (point-min) (point-max))
-			(insert "<input id=\"" id "\" name=\"" name "\""
-				(if default
-				    (concat " value=\"" default "\" ")
-				  " ")
-				val
-				;; Fit the width to that of the select form.
-				" size=\"13\">"))
-		    (goto-char (point-max)))))
-	    (goto-char nd))
-	(if (match-beginning 2)
-	    (save-restriction
-	      (narrow-to-region st nd)
-	      (delete-region (goto-char (match-beginning 2)) (match-end 2))
-	      (insert val)
-	      (goto-char (point-max)))
-	  (goto-char nd))))))
+				  (goto-char (match-end 1))
+				  (skip-chars-backward "\t\n ")
+				  (buffer-substring (match-beginning 1)
+						    (point))))
+			  (delete-region (point-min) (point-max))
+			  (insert "<input id=\"" id "\" name=\"" name "\""
+				  (if default
+				      (concat " value=\"" default "\" ")
+				    " ")
+				  val
+				  ;; Fit the width to that of the select form.
+				  " size=\"13\">"))
+		      (goto-char (point-max)))))
+	      (goto-char nd))
+	  (if (match-beginning 1)
+	      (save-restriction
+		(narrow-to-region st nd)
+		(delete-region (goto-char (match-beginning 1)) (match-end 1))
+		(insert val)
+		(goto-char (point-max)))
+	    (goto-char nd)))))))
 
 (defun w3m-filter-fix-tfoot-rendering (url &optional recursion)
   "Render <tfoot>...</tfoot> after <tbody>...</tbody>."
@@ -691,7 +783,7 @@ href=\"#\\([a-z][-.0-9:_a-z]*\\)\"" nil t)
 	  (goto-char (point-min))
 	  (insert (format "<meta HTTP-EQUIV=\"Refresh\" CONTENT=\"0;URL=%s\">\n"
 			  aturl)))
-      (while (re-search-forward (concat "<a[ \t\r\l\n]+href=\"javascript:[^(]+('"
+      (while (re-search-forward (concat "<a[ \t\r\f\n]+href=\"javascript:[^(]+('"
 					"\\([^']+\\)')\">")
 				nil t)
 	(setq aturl (match-string 1))
@@ -702,11 +794,280 @@ href=\"#\\([a-z][-.0-9:_a-z]*\\)\"" nil t)
   "Make anchor reference to work."
   (goto-char (point-min))
   (let (matched-text refid)
-    (while (re-search-forward 
+    (while (re-search-forward
 	    "<\\(?:sup\\|cite\\) id=\"\\([^\"]*\\)\"" nil t)
       (setq matched-text (match-string 0)
 	    refid        (match-string 1))
       (delete-region (match-beginning 0) (match-end 0))
       (insert (format "<a name=\"%s\"></a>%s" refid matched-text)))))
 
+(defun w3m-filter-rt (url)
+  "Filter top and bottom cruft for rt.com."
+  (if (w3m-filter-delete-regions url
+				 "<body.*>"
+				 "<h1.*>"
+				 t t t nil nil 1)
+      (w3m-filter-delete-regions url "<div class=\"news-block\"" "</body>")
+    ;; BEGIN: When page is Home or Article Index pages ...
+    (w3m-filter-delete-regions url
+			       "<body>"
+			       "<div class=\"header__date\">"
+			       nil nil nil nil nil 1)
+    (w3m-filter-delete-regions url
+			       "</time>"
+			       "<div class=\"layout__content \">"
+			       nil nil nil (point) nil 1)
+    (w3m-filter-delete-regions url
+			       "<div class=\"rightpoll\">"
+			       "</body>"
+			       nil nil nil (point) nil 1)
+    (goto-char (point-min))
+    (when (search-forward "<h2 class=\"main-viralbox__title\">" nil t)
+      (let ((p (match-beginning 0)))
+	(goto-char (point-min))
+	(while (re-search-forward "<a[^>]+>\\(<img[^>]+>\\)</a>" p t)
+	  (replace-match "\\1")))
+      (while (re-search-forward "<a[^>]+>\\(<div[^>]+>\\)?\
+\\(<img[^>]+>\\)\\(.*?\\)\\(<a[^<]+</a>\\)"
+				nil t)
+	(replace-match "\\3\\4 \\2")))
+    (w3m-filter-delete-regions url
+			       "<button"
+			       "</button>")
+    (goto-char (point-min))
+    (while (re-search-forward "<h2[^>]*>" nil t)
+      (replace-match "<b>")
+      (when (search-forward "</h2>" nil t)
+	(replace-match "</b>"))
+      t)
+    (goto-char (point-min))
+    (when (search-forward "\
+<strong class=\"news-line__title\">News line</strong>\
+<div class=\"news-line__content\">"
+			  nil t)
+      (replace-match
+       "<br><a href=\"https://www.rt.com/news/line/\">News Line:</a>")
+      (when (re-search-forward "<ul[^>]+>" nil t)
+	(replace-match ""))
+      (w3m-filter-replace-regexp url
+				 "<li class=\"news-line__item\">[^>]+>"
+				 "<br>* "
+				 (point))
+      (w3m-filter-delete-regions url "<ul class=\"buttons\">" "</ul>")
+      (w3m-filter-delete-regions url
+				 "<div class=\"banners__border\">" "</div>")))
+  ;; END: When page is Home or Article Index pages ...
+  (goto-char (point-min))
+  (while (re-search-forward "alt=\"[^\"]+\"" nil t)
+    (replace-match "alt=[photo]"))
+  (goto-char (point-min))
+  (when (re-search-forward "</h1>.*me:" nil t)
+    (replace-match "</b><blockquote>Published:"))
+  (when (search-forward
+	 "</div><div class=\"article__short-url\"><div class=\"short-url\">"
+	 nil t)
+    (replace-match ""))
+  (when (search-forward "Get short URL" nil t)
+    (replace-match " [Short URL]"))
+  (when (search-forward
+	 "</div></div><div class=\"article__cover\"><div class=\"media  \">"
+	 nil t)
+    (replace-match " "))
+  (when (w3m-filter-delete-regions url
+				   "<div class=\"article__share\">"
+				   "<div class=\"article__summary summary \">"
+				   nil nil nil (point) nil 1)
+    (insert "</blockquote><p>"))
+  (goto-char (point-min))
+  (while (search-forward "</p>&mdash;" nil t)
+    (replace-match "<br>&mdash;"))
+  (goto-char (point-min))
+  (while (search-forward "<center>" nil t)
+    (replace-match ""))
+  (goto-char (point-min))
+  (while (re-search-forward "\\(<a [^>]+>\\)?<strong>\\(<a [^>]+>\\)?\
+READ MORE:\\([^<]+\\)\\(</a>\\)?</strong>\\(</p>\\)?"
+			    nil t)
+    (replace-match
+     "<blockquote>Related Article:<br>\\1\\2\\3</a></blockquote>"))
+  (goto-char (point-min))
+  (while (search-forward "</a><a href" nil t)
+    (replace-match "</a> - <a href"))
+  (goto-char (point-min))
+  (while (re-search-forward "\\(<a[^>]+>\\) *\n+  *\\(<img[^>]+>\\)" nil t)
+    (replace-match "\\2<br>\\1"))
+  (goto-char (point-min))
+  (while (search-forward "<div class=\"tags-trends\">" nil t)
+    (replace-match "<blockquote>")
+    (when (search-forward "Trends<" nil t)
+      (replace-match "Trends: <"))
+    (when (search-forward "</div>" nil t)
+      (replace-match "</blockquote>")))
+  (goto-char (point-min))
+  (while (search-forward "<div class=\"read-more__title\">" nil t)
+    (replace-match "<blockquote>\\1")
+    (when (re-search-forward "Read more:?")
+      (replace-match "Related article:"))
+    (when (search-forward "</a>")
+      (replace-match "\\1</blockquote>")))
+  (goto-char (point-min))
+  (while (re-search-forward "\\(<div[^>]+>\\)+" nil t)
+    (replace-match "")))
+
+(defun w3m-filter-slashdot (url)
+  "Filter js deadlinks, top and bottom cruft for slashdot"
+  (goto-char (point-min))
+  ;; BEGIN: When page is Home or Article Index pages ...
+  (when (search-forward "<body class=\"anon index" nil t)
+    ;; Title pages have css rule to treat <i> as <blockquote>
+    (w3m-filter-replace-regexp url "<i>" "<blockquote>")
+    (w3m-filter-replace-regexp url "</i>" "</blockquote>")
+    (goto-char (point-min))
+    ;; Title page tighten display
+    (while (re-search-forward "<h2[^>]+>" nil t)
+      (replace-match "<p><b>"))
+    (goto-char (point-min))
+    (search-forward "<!-- WIT -->" nil t)
+    (w3m-filter-delete-regions url
+			       "<!-- WIT -->"
+			       "<header id=\"mostdiscussed-title\">"
+			       nil nil nil (point) nil 1)
+    (w3m-filter-delete-regions url
+			       "<header id=\"srandblock-title\">"
+			       "<nav role=\"firehose footer\">"
+			       nil nil nil (point) nil 1)
+    (insert "Jump to stories from:<br>&nbsp; ")
+    (search-forward "<ul id=\"pagination-controls\">" nil t)
+    (replace-match "")
+    (w3m-filter-replace-regexp url
+			       "<li class=\"fleft\">"
+			       ""
+			       (point) nil 8)
+    (w3m-filter-delete-regions url
+			       "</li>"
+			       "</body>"
+			       nil t nil (point) nil 1)
+    (goto-char (point-min))
+    (while (w3m-filter-delete-regions url
+				      "<div class=\"story-tags\">"
+				      "<span id=\"tagbar-.+>"
+				      nil nil t (point) nil 1)
+      (insert "<br>Tags: "))
+    (goto-char (point-min))
+    (while (w3m-filter-delete-regions url
+				      "<div class=\"tag-menu\">"
+				      "</footer>"
+				      nil nil nil (point) nil 1)
+      (insert "<hr><p>")))
+  ;; END: When page is Home or Article Index pages ...
+  (w3m-filter-delete-regions url
+			     "<body"
+			     "<header>"
+			     nil nil nil nil nil 1)
+  (insert "<body>")
+  (w3m-filter-replace-regexp url
+			     "<div class=\"body\"[^>]+>"
+			     "<p>"
+			     (point))
+  (goto-char (point-min))
+  (while (w3m-filter-delete-regions url
+				    "<!-- comment bubble -->"
+				    "<span class=\"story-byline\">"
+				    nil nil nil (point) nil 1)
+    (insert "</b><br>&nbsp;&nbsp;&nbsp;&nbsp;")
+    (when (search-forward "</time>" nil t)
+      (replace-match "<br>&nbsp;&nbsp;&nbsp;&nbsp;"))
+    t)
+  (goto-char (point-min))
+  (when (search-forward "<aside class=\"grid_24 view_mode\">" nil t)
+    (w3m-filter-replace-regexp url
+			       "<i>"
+			       ""
+			       nil (match-beginning 0)))
+  (when (w3m-filter-delete-regions url
+				   "<aside class=\"grid_24 view_mode\">"
+				   "<span class=\"totalcommentcnt\">"
+				   nil nil t (point) nil 1)
+    (insert "<br><b>"))
+  (when (w3m-filter-delete-regions url
+				   "</a>"
+				   "<ul id=\"commentlisting\" class=\"d2\">"
+				   nil nil t (point) nil 1)
+    (insert "</b><ul>"))
+  (w3m-filter-delete-regions url
+			     "<div class=\"grid_10 d1or2\""
+			     "<ul id=\"commtree_"
+			     nil t t (point) nil 1)
+  (w3m-filter-replace-regexp url
+			     "<div id=\"comment_body_[^>]+> *<p>"
+			     "")
+  (w3m-filter-replace-regexp url
+			     "<div class=\"commentSub\""
+			     "<p>\\&")
+  (w3m-filter-replace-regexp url
+			     "<div class=\"quote\"><p>\\(.+?\\)</div>"
+			     "<blockquote>\\1</blockquote>")
+  (w3m-filter-delete-regions url
+			     "(<span class=\"ind\""
+			     "</a>)")
+  (w3m-filter-delete-regions url
+			     "<div class=\"commentSub\""
+			     "<div id=\"replyto_[0-9]+\">" nil nil t)
+  (w3m-filter-delete-regions url
+			     "<a id=\"reply_link_"
+			     "Flag as Inappropriate</a>")
+  (w3m-filter-delete-regions url
+			     "<noscript><p><b>There may be more comments"
+			     "</body>")
+  (goto-char (point-min))
+  (while (re-search-forward "<ul id=\"group_[^>]+>\n\t+\
+<li id=\"hiddens_[0-9]+\" class=\"hide\"></li>\n</ul>"
+			    nil t)
+    (replace-match ""))
+  (goto-char (point-min))
+  (while (re-search-forward "<li id=\"tree_[0-9]+\" class=\"comment hidden\">"
+			    nil t)
+    (replace-match ""))
+  (goto-char (point-min))
+  (while (re-search-forward "<div[^>]+>\n*</div>" nil t)
+    (replace-match ""))
+  (goto-char (point-min))
+  (while (re-search-forward "<span id=\"comment_score_.+:</span>" nil t)
+    (replace-match "</b>(Score:"))
+  (goto-char (point-min))
+  (while (re-search-forward "</a> *<span class=\"uid\"><a[^>]+>" nil t)
+    (replace-match ""))
+  (goto-char (point-min))
+  (while (search-forward "<h4>" nil t)
+    (replace-match "<b>")
+    (when (search-forward "</h4>" nil t)
+      (replace-match "</b><br>"))
+    t)
+  (w3m-filter-delete-regions url "<article" "<header>")
+  (goto-char (point-min))
+  (while (re-search-forward
+	  "\\( beneath your current threshold.\\)</b></noscript>"
+	  nil t)
+    (replace-match "</b>\\1<p></noscript>"))
+  (w3m-filter-replace-regexp url "</a>" "</a> ")
+  (goto-char (point-min))
+  (while (re-search-forward "\n[\n\t ]+" nil t)
+    (replace-match "")))
+
+(defun w3m-filter-geocities-remove-garbage (url)
+  "Remove garbage in http://www.geocities.co.jp/*."
+  (w3m-filter-delete-regions
+   url
+   "<DIV ALIGN=CENTER>\n<!--*/GeoGuide/*-->" "<!--*/GeoGuide/*-->\n</DIV>"))
+
+(defun w3m-filter-infoseek-remove-ads (url)
+  "Remove ADV in http://*.hp.infoseek.co.jp/*."
+  (w3m-filter-delete-regions url "<!-- start AD -->" "<!-- end AD -->"))
+
+(defun w3m-filter-ascii24-remove-ads (url)
+  "Remove ADV in http://linux.ascii24.com/linux/*."
+  (w3m-filter-delete-regions
+   url
+   "<!-- DAC CHANNEL AD START -->" "<!-- DAC CHANNEL AD END -->"))
+
 ;;; w3m-filter.el ends here
