@@ -1,6 +1,6 @@
 ;;; w3m-proc.el --- Functions and macros to control sub-processes
 
-;; Copyright (C) 2001-2005, 2007-2010, 2012
+;; Copyright (C) 2001-2005, 2007-2010, 2012, 2013, 2016, 2017
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -44,26 +44,23 @@
 (require 'w3m-util)
 
 (eval-when-compile
-  ;; Variable(s) which are used in the following inline functions.
-  ;; They should be defined in the other module at run-time.
-  (defvar w3m-current-url)
-  (defvar w3m-current-buffer)
-  (defvar w3m-current-process)
-  (defvar w3m-profile-directory)
-  (defvar w3m-terminal-coding-system)
+  ;; External functions and variables used in this module,
+  ;; and should be defined in the other module at run-time.
+  (autoload 'w3m-idle-images-show-unqueue "w3m")
+  (defvar w3m-async-exec)
+  (defvar w3m-clear-display-while-reading)
   (defvar w3m-command)
   (defvar w3m-command-arguments)
   (defvar w3m-command-environment)
-  (defvar w3m-async-exec)
+  (defvar w3m-current-buffer)
+  (defvar w3m-current-process)
+  (defvar w3m-current-title)
+  (defvar w3m-current-url)
   (defvar w3m-process-connection-type)
   (defvar w3m-process-modeline-format)
-  (defvar w3m-work-buffer-list)
-  (autoload 'w3m-idle-images-show-unqueue "w3m"))
-
-;; Silence the Emacs's byte-compiler that says ``might not be defined''.
-(eval-when-compile
-  (defun w3m-decode-coding-string-with-priority (str coding)
-    ()))
+  (defvar w3m-profile-directory)
+  (defvar w3m-terminal-coding-system)
+  (defvar w3m-work-buffer-list))
 
 (defvar w3m-process-inhibit-quit t
   "`w3m-process-sentinel' binds `inhibit-quit' according to this variable.")
@@ -241,7 +238,7 @@ generated asynchronous process is ignored.  Otherwise,
 (defun w3m-process-start-queued-processes ()
   "Start a process which is registerd in `w3m-process-queue' if the
 number of current working processes is less than `w3m-process-max'."
-  (w3m-process-kill-stray-processes)
+  ;; Run `w3m-process-kill-stray-processes' in advance.
   (let ((num 0))
     (catch 'last
       (dolist (obj (reverse w3m-process-queue))
@@ -297,7 +294,17 @@ which have no handler."
   (when (buffer-name buffer)
     (with-current-buffer buffer
       (setq w3m-current-process nil)))
-  (w3m-process-start-queued-processes)
+  (w3m-process-kill-stray-processes)
+  (if w3m-process-queue
+      (w3m-process-start-queued-processes)
+    (when (and w3m-clear-display-while-reading
+	       (progn
+		 (goto-char (point-min))
+		 (looking-at "\n* *Reading [^\n]+\\.\\.\\.\\'")))
+      (let ((inhibit-read-only t))
+	(erase-buffer)
+	(setq w3m-current-url nil
+	      w3m-current-title nil))))
   (w3m-force-window-update-later buffer))
 
 (defun w3m-process-shutdown ()
@@ -518,8 +525,7 @@ evaluated in a temporary buffer."
    ((not exit-status)
     (setq w3m-process-exit-status nil))
    (t
-    (setq w3m-process-exit-status
-	  (string-as-multibyte (format "%s" exit-status)))
+    (setq w3m-process-exit-status (format "%s" exit-status))
     nil)))
 
 (defvar w3m-process-background nil
@@ -579,6 +585,7 @@ evaluated in a temporary buffer."
 		(throw 'last nil)))))
       (delete-process process)
       (unless ignore-queue
+	(w3m-process-kill-stray-processes)
 	(w3m-process-start-queued-processes)))))
 
 (defun w3m-process-filter (process string)
