@@ -1,6 +1,6 @@
 ;;; w3m-util.el --- Utility macros and functions for emacs-w3m
 
-;; Copyright (C) 2001-2014, 2016 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001-2014, 2016, 2017 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;          Shun-ichi GOTO     <gotoh@taiyo.co.jp>,
@@ -73,6 +73,8 @@
 
 ;;; Things should be defined in advance:
 (eval-when-compile
+  (autoload 'w3m-detect-coding-region
+    (if (featurep 'emacs) "w3m-ems" "w3m-xmas"))
   (autoload 'w3m-fb-frame-parameter "w3m-fb")
   (autoload 'w3m-history-restore-position "w3m-hist" nil t))
 
@@ -133,72 +135,19 @@ or `debug-on-quit' is non-nil."
   "Decode the string STR which is encoded in CODING.
 If CODING is a list, look for the coding system using it as a priority
 list."
-  (w3m-static-cond
-   ((featurep 'emacs)
-    (setq str (string-make-unibyte str))
-    (when (listp coding)
-      (setq coding
-	    (with-temp-buffer
-	      (set-buffer-multibyte nil)
-	      (insert str)
-	      ;; A copy of `w3m-detect-coding-region' defined in w3m-ems.el.
-	      (w3m-static-if (fboundp 'with-coding-priority)
-		  (with-coding-priority coding
-		    (car (detect-coding-region (point-min) (point-max))))
-		(let (category categories)
-		  (dolist (codesys coding)
-		    (setq category (coding-system-category codesys))
-		    (unless (or (null category) (assq category categories))
-		      (push (cons category codesys) categories)))
-		  (car (detect-coding-with-priority
-			(point-min) (point-max) (nreverse categories))))))))
-    (decode-coding-string str
-			  (or coding
-			      w3m-default-coding-system
-			      w3m-coding-system
-			      'iso-2022-7bit)))
-   ;; XEmacs w/ Mule
-   ((featurep 'mule)
-    (if (listp coding)
-	(with-temp-buffer
-	  (insert str)
-	  (let* ((orig-category-list (coding-priority-list))
-		 (orig-category-systems (mapcar #'coding-category-system
-						orig-category-list))
-		 codesys category priority-list)
-	    (unwind-protect
-		(progn
-		  (while coding
-		    (setq codesys (car coding)
-			  coding (cdr coding)
-			  category (or (coding-system-category codesys)
-				       (coding-system-name codesys)))
-		    (unless (or (eq (coding-system-type codesys) 'undecided)
-				(assq category priority-list))
-		      (set-coding-category-system category codesys)
-		      (push category priority-list)))
-		  (set-coding-priority-list (nreverse priority-list))
-		  ;; `detect-coding-region' always returns `undecided'
-		  ;; ignoring `priority-list' in XEmacs 21.5-b19, but
-		  ;; that's okay.
-		  (when (consp (setq codesys (detect-coding-region
-					      (point-min) (point-max))))
-		    (setq codesys (car codesys)))
-		  (decode-coding-region (point-min) (point-max)
-					(or codesys
-					    w3m-default-coding-system
-					    w3m-coding-system
-					    'iso-2022-7bit))
-		  (buffer-string))
-	      (set-coding-priority-list orig-category-list)
-	      (while orig-category-list
-		(set-coding-category-system (car orig-category-list)
-					    (car orig-category-systems))
-		(setq orig-category-list (cdr orig-category-list)
-		      orig-category-systems (cdr orig-category-systems))))))
-      (decode-coding-string str coding)))
-   ;; XEmacs w/o Mule
-   (t str)))
+  (w3m-static-if (featurep 'mule)
+      (with-temp-buffer
+	(w3m-static-when (featurep 'emacs) (set-buffer-multibyte nil))
+	(insert str)
+	(decode-coding-string
+	 (buffer-string)
+	 (or (if (listp coding)
+		 (w3m-detect-coding-region (point-min) (point-max) coding)
+	       coding)
+	     w3m-default-coding-system
+	     w3m-coding-system
+	     'iso-2022-7bit)))
+    str))
 
 ;;; Text props:
 
@@ -494,17 +443,36 @@ An argument of nil means kill the current buffer."
 	(setq newname (format "%s *w3m*<%d>" (w3m-current-title) number)))
       (rename-buffer newname))))
 
-(defun w3m-generate-new-buffer (name)
-  (if w3m-use-title-buffer-name
-      (let* ((maxbuf (let ((w3m-fb-mode nil))
-		       (car (nreverse (w3m-list-buffers)))))
-	     (number (w3m-buffer-number maxbuf)))
-	(when (string-match "\\*w3m\\*\\(<\\([0-9]+\\)>\\)?\\'" name)
-	  (setq name "*w3m*"))
-	(if (and maxbuf number)
-	    (generate-new-buffer (format "%s<%d>" name (1+ number)))
-	  (generate-new-buffer name)))
-    (generate-new-buffer name)))
+(defun w3m-generate-new-buffer (name &optional next)
+  "Create and return a buffer with a name based on NAME.
+Make the new buffer the next of the current buffer if NEXT is non-nil."
+  (when (string-match "<[0-9]+>\\'" name)
+    (setq name (substring name 0 (match-beginning 0))))
+  (let* ((w3m-fb-mode nil)
+	 (buffers (w3m-list-buffers))
+	 (regexp (concat "\\`" (regexp-quote name) "\\(?:<[0-9]+>\\)?\\'"))
+	 (siblings (delq nil
+			 (mapcar
+			  (lambda (buffer)
+			    (when (string-match regexp (buffer-name buffer))
+			      buffer))
+			  buffers)))
+	 youngers cur number num)
+    (if (and next
+	     (setq youngers (cdr (memq (setq cur (current-buffer)) siblings))))
+	(progn
+	  (setq number (1+ (w3m-buffer-number cur))
+		num (+ 1 (length youngers)
+		       (w3m-buffer-number (car (reverse youngers)))))
+	  (dolist (buffer (reverse youngers))
+	    (w3m-buffer-set-number buffer (setq num (1- num))))
+	  (setq num number)
+	  (dolist (buffer youngers)
+	    (w3m-buffer-set-number buffer (setq num (1+ num))))
+	  (generate-new-buffer (format "%s<%d>" name number)))
+      (if (setq number (w3m-buffer-number (car (nreverse siblings))))
+	  (generate-new-buffer (format "%s<%d>" name (1+ number)))
+	(generate-new-buffer name)))))
 
 (defun w3m-buffer-name-lessp (x y)
   "Return t if first arg buffer's name is less than second."
@@ -680,10 +648,12 @@ according to `w3m-pop-up-windows' and `w3m-pop-up-frames' (which see)."
 	      (w3m-static-when (featurep 'xemacs)
 		(focus-frame frame))))
 	;; Simply switch to BUFFER in the current frame.
-	(if (w3m-popup-window-p)
-	    (let ((pop-up-windows t))
-	      (pop-to-buffer buffer))
-	  (switch-to-buffer buffer))
+	(let ((cd default-directory))
+	  (if (w3m-popup-window-p)
+	      (let ((pop-up-windows t))
+		(pop-to-buffer buffer))
+	    (switch-to-buffer buffer))
+	  (setq default-directory cd))
 	(w3m-history-restore-position)))))
 
 (eval-when-compile
@@ -758,53 +728,53 @@ objects will not be deleted:
 	(set-buffer buffer)
 	(while windows
 	  (setq window (pop windows)
-		frame (window-frame window))
+		frame (and (window-live-p window) (window-frame window)))
 	  (when (and frame
 		     (not (eq frame exception)))
+	    (setq flag nil)
 	    (setq one-window-p
-		  (w3m-static-if (featurep 'xemacs)
-		      (one-window-p t frame)
-		    ;; Emulate XEmacs version's `one-window-p'.
-		    (prog2
-			(setq flag nil)
-			(catch 'exceeded
-			  (walk-windows (lambda (w)
-					  (when (eq (window-frame w) frame)
-					    (if flag
-						(throw 'exceeded nil)
-					      (setq flag t))))
-					'no-minibuf t)
-			  flag)
-		      (set-buffer buffer))))
-	    (if (and (memq frame w3m-initial-frames)
-		     (not (eq (next-frame frame) frame)))
-		(if (or
-		     ;; A frame having the sole window can be deleted.
-		     one-window-p
-		     ;; Also a frame having only windows for emacs-w3m
-		     ;; sessions or the buffer selection can be deleted.
-		     (progn
-		       (setq flag t)
-		       (walk-windows
-			(lambda (w)
-			  (when flag
-			    (if (eq w exception)
-				(setq flag nil)
-			      (set-buffer (window-buffer w))
-			      (setq flag (or (memq major-mode
-						   '(w3m-mode
-						     w3m-select-buffer-mode
-						     w3m-session-select-mode))
-					     (string-match "\\` ?\\*w3m[ -]"
-							   (buffer-name)))))))
-			'no-minibuf)
-		       (set-buffer buffer)
-		       flag))
-		    (delete-frame frame)
-		  (delete-window window))
-	      ;; do not clear the layout.
-	      (switch-to-buffer nil))))))))
-
+		  ;; This is similar to the `one-window-p' function
+		  ;; but works for even unselected windows in addition
+		  ;; to the selected window in all frames.
+		  (catch 'two
+		    (walk-windows (lambda (w)
+				    (when (eq (window-frame w) frame)
+				      (if flag
+					  (throw 'two nil)
+					(setq flag t))))
+				  'no-minibuf t)
+		    flag))
+	    (when (and
+		   (or
+		    ;; A frame having only windows for emacs-w3m
+		    ;; sessions or the buffer selection can be deleted.
+		    (progn
+		      (setq flag t)
+		      (walk-windows
+		       (lambda (w)
+			 (when flag
+			   (if (eq w exception)
+			       (setq flag nil)
+			     (set-buffer (window-buffer w))
+			     (when (setq flag
+					 (or
+					  (memq major-mode
+						'(w3m-mode
+						  w3m-select-buffer-mode
+						  w3m-session-select-mode))
+					  (string-match "\\` ?\\*w3m[ -]"
+							(buffer-name))))
+			       (if (eq (next-window w 'no-minibuf) w)
+				   (bury-buffer)
+				 (delete-window w))))))
+		       'no-minibuf)
+		      (set-buffer buffer)
+		      flag)
+		    ;; Also a frame having the sole window can be deleted.
+		    one-window-p)
+		   (memq frame w3m-initial-frames)
+		   (not (eq (next-frame frame) frame)))
+	      (delete-frame frame))))))))
 
 ;;; Navigation:
 
@@ -852,8 +822,24 @@ If currently over such PROPERTY, find next such occurence."
 
 ;;; Miscellaneous:
 
-(defconst w3m-url-fallback-base "http:///")
-(defconst w3m-url-invalid-regexp "\\`http:///")
+(eval-and-compile
+  (defconst w3m-url-invalid-base "http:///"
+    "A url base used to make url absolutely invalid.
+`w3m-expand-url' will use it as a last resort if no other appropriate
+base is given."))
+
+(defsubst w3m-url-valid (url)
+  "Return URL if it is not marked invalid, otherwise nil.
+This function is intended only to reject a url that `w3m-expand-url'
+marks invalid purposely (using `w3m-url-invalid-base'), does not
+necessarily guarantee that URL to return is valid in a general sense."
+  ;; cf. [emacs-w3m:04095], [emacs-w3m:04101] (Japanese),
+  ;; and [emacs-w3m:12761] (summarized in English).
+  (and url
+       (not (string-match (eval-when-compile
+			    (concat "\\`" w3m-url-invalid-base))
+			  url))
+       url))
 
 (defmacro w3m-substitute-key-definitions (new-map old-map &rest keys)
   "In NEW-MAP substitute cascade of OLD-MAP KEYS.
@@ -869,10 +855,6 @@ KEYS is alternating list of key-value."
 	     (setq keys (cddr keys)))
 	   (nreverse res)))))
 
-(defun w3m-url-valid (url)
-  (and url (not (string-match w3m-url-invalid-regexp url))
-       url))
-
 (defmacro w3m-set-match-data (list)
   "Same as the `set-match-data'; convert points into markers under XEmacs."
   (if (featurep 'xemacs)
@@ -1364,12 +1346,11 @@ pairs from PLIST whose value is nil."
 (defmacro w3m-insert-string (string)
   "Insert STRING at point without conversions in either case the
 multibyteness of the buffer."
-  (if (and (fboundp 'string-as-multibyte)
-	   (subrp (symbol-function 'string-as-multibyte)))
+  (if (featurep 'emacs)
       `(let ((string ,string))
 	 (insert (if enable-multibyte-characters
-		     (string-as-multibyte string)
-		   (string-as-unibyte string))))
+		     string
+		   (encode-coding-string string 'utf-8-emacs))))
     `(insert ,string)))
 
 (defun w3m-custom-hook-initialize (symbol value)
@@ -1614,6 +1595,7 @@ the function cell of FUNCs rather than their value cell.
 	 ,@body)
     `(flet ,bindings ,@body)))
 (put 'w3m-flet 'lisp-indent-function 1)
+(def-edebug-spec w3m-flet ((&rest (sexp sexp &rest form)) &rest form))
 
 (defmacro w3m-labels (bindings &rest body)
   "Make temporary function bindings.
@@ -1626,6 +1608,7 @@ rather than relying on `lexical-binding'.
   `(,(progn (require 'cl) (if (fboundp 'cl-labels) 'cl-labels 'labels))
     ,bindings ,@body))
 (put 'w3m-labels 'lisp-indent-function 1)
+(def-edebug-spec w3m-labels ((&rest (sexp sexp &rest form)) &rest form))
 
 (eval-when-compile (require 'wid-edit))
 (defun w3m-widget-type-convert-widget (widget)
