@@ -1,6 +1,6 @@
 ;;; w3mhack.el --- a hack to setup the environment for building w3m
 
-;; Copyright (C) 2001-2010, 2012
+;; Copyright (C) 2001-2010, 2012, 2013, 2015, 2017
 ;; TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Author: Katsumi Yamaoka <yamaoka@jpl.org>
@@ -71,6 +71,7 @@ nil means that all icons are installed to the default directory.")
 but do not execute them.")
 
 (require 'cl)
+(condition-case nil (require 'seq) (error))
 
 ;; Check whether the shell command can be used.
 (let ((test (lambda (shell)
@@ -289,7 +290,7 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
 	 (version-specific-modules '("w3m-ems.el" "w3m-xmas.el"))
 	 (ignores;; modules not to be byte-compiled.
 	  (append
-	   (list "w3mhack.el" "w3m-setup.el" w3mhack-load-file)
+	   (list "w3mhack.el")
 	   (delete (if (featurep 'xemacs) "w3m-xmas.el" "w3m-ems.el")
 		   (copy-sequence version-specific-modules))))
 	 (shimbun-dir (file-name-as-directory shimbun-module-directory))
@@ -367,6 +368,23 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
   (dolist (module (w3mhack-module-list))
     (princ (format "%sc " module))))
 
+(if (featurep 'emacs)
+    (defun w3mhack-compile-file (file)
+      "Byte-compile FILE after reporting that it's being compiled."
+      ;; The byte compiler in Emacs >=25 doesn't say much.
+      (message "Compiling %s..." (file-name-nondirectory file))
+      (if (string-equal (file-name-nondirectory file) "w3m-filter.el")
+	  ;; Silence the warning "value returned from (> i 0) is unused".
+	  ;; See `w3m-filter-delete-regions', `w3m-filter-replace-regexp',
+	  ;; and the filter functions that use those macros.
+	  (let ((val (get '> 'side-effect-free)))
+	    (put '> 'side-effect-free 'error-free)
+	    (unwind-protect
+		(byte-compile-file file)
+	      (put '> 'side-effect-free val)))
+	(byte-compile-file file)))
+  (defalias 'w3mhack-compile-file 'byte-compile-file))
+
 (defun w3mhack-compile ()
   "Byte-compile the w3m modules."
   (w3mhack-generate-load-file)
@@ -382,7 +400,7 @@ Error: You have to install APEL before building emacs-w3m, see manuals.
     (setq modules (nreverse modules))
     (while modules
       (condition-case nil
-	  (byte-compile-file (car modules))
+	  (w3mhack-compile-file (car modules))
 	(error))
       (setq modules (cdr modules)))))
 
@@ -612,8 +630,12 @@ to remove some obsolete variables in the first argument VARLIST."
 	;; This system doesn't allow hard links.
 	(setq make-hardlink 'copy-file))
       (dolist (el (cons w3mhack-load-file (w3mhack-module-list)))
-	(funcall make-hardlink
-		 el (expand-file-name (file-name-nondirectory el) temp-dir)))
+	(setq hardlink (expand-file-name (file-name-nondirectory el) temp-dir))
+	(or (file-exists-p hardlink)
+	    ;; w3m-load.el may have been already generated.
+	    (funcall make-hardlink
+		     el (expand-file-name (file-name-nondirectory el)
+					  temp-dir))))
       (with-temp-buffer
 	(let ((standard-output (current-buffer)))
 	  (Custom-make-dependencies temp-dir))
@@ -857,10 +879,8 @@ NOTE: This function must be called from the top directory."
 					       ',coding)))))
 			       (t
 				`(lambda (fmt &rest args)
-				   (funcall ,si:message "%s"
-					    (encode-coding-string
-					     (apply 'format fmt args)
-					     ',coding))))))))
+				   (let ((coding-system-for-write ',coding))
+				     (apply ,si:message fmt args))))))))
 		     ((featurep 'xemacs)
 		      (byte-compile
 		       `(lambda (fmt &rest args)
@@ -979,6 +999,14 @@ NOTE: This function must be called from the top directory."
 	  (goto-char (point-max))
 	  (insert (if (featurep 'xemacs) "\C-l\n" "")
 		  ";;; " w3mhack-load-file " ends here\n"))
+
+	;; Delete "no-byte-compile" so that w3m-load.el will byte compile.
+	;; Byte compiling gives dynamic docstrings which means they aren't
+	;; loaded into memory unless or until required.
+	(goto-char (point-min))
+	(when (re-search-forward "^;; no-byte-compile: t\n" nil t)
+	  (delete-region (match-beginning 0) (match-end 0)))
+
 	(save-buffer)))))
 
 (defun w3mhack-generate-xemacs-load-file (file)
